package MyApp::Model::Friend;
use strict;
use warnings;

use Mouse;

extends "MyApp::Model::Base";

has [qw/user_a user_b/] => (
    is => "rw",
    isa => "MyApp::Model::User",
);

no Mouse;

sub build {
    my ($self, %args) = @_;
    my $user_a = $args{user_a};
    my $user_b = $args{user_b};

    $self->user_a($user_a);
    $self->user_b($user_b);
    $self->_fetched(1);
    $self->_build;
}

sub _build {
    my $self = shift;

    my $txn = $self->dbh->txn_scope;
    my @binds = (
        $self->user_a->id, $self->user_b->id,
        $self->user_b->id, $self->user_a->id,
    );
    $self->dbh->query(<<SQL, @binds);
INSERT INTO `friends` (user_a_id, user_b_id)
    VALUES (?, ?), (?, ?);
SQL

    $txn->commit;
}

sub info {
    my $self = shift;

    return {
        friend => {
            user_a => $self->user_a->info,
            user_b => $self->user_b->info,
        },
    };
}

1;
