package MyApp::Model::Friends;
use strict;
use warnings;

use MyApp::Model::User;
use Mouse;

extends "MyApp::Model::Base";

has [qw/user/] => (
    is => "ro",
    isa => "MyApp::Model::User",
);

has [qw/friend_users/] => (
    is => "rw",
    isa => "ArrayRef[MyApp::Model::User]",
);

no Mouse;

sub new_by_user {
    my ($class, $user) = @_;

    my $friends = $class->new(user => $user);

    return $friends;
}

sub _fetch {
    my $self = shift;

    return if $self->_fetched;

    my $friend_user_rows = $self->dbh->select_all(<<SQL, $self->user->id);
SELECT * FROM users WHERE id IN (SELECT user_b_id FROM friends WHERE user_a_id = ?);
SQL
    my @friend_users = map {
        MyApp::Model::User->new_by_hashref($_)
    } @$friend_user_rows;
    $self->friend_users(\@friend_users);
    $self->_fetched(1);
}

sub info {
    my $self = shift;
    $self->_fetch;

    my @friend_info_keys = qw/id name/;

    my @friend_users_info;
    for my $friend_user (@{$self->friend_users}) {
        push @friend_users_info, {
            map { $_ => $friend_user->{$_} } @friend_info_keys
        };
    }

    return {
        friends => \@friend_users_info,
    };
};

1;
