package MyApp::Model::Item;
use strict;
use warnings;

use JSON;
use MyApp::Model::UserItem;
use MyApp::Model::ItemEffect::RecoverEnergy;
use Mouse;

extends "MyApp::Model::Base";

has id => (
    is => "rw",
    isa => "Int",
);

has name => (
    is => "rw",
    isa => "Str",
);

has effect_type => (
    is => "rw",
    isa => "Int",
);

has effect_parameter => (
    is => "rw",
    isa => "HashRef",
);

no Mouse;

sub build {
    my ($self, %args) = @_;
    my $item_id = $args{item_id};

    $self->id($item_id);
    $self->_fetch;
}

my $json = JSON->new->utf8;

sub _fetch {
    my $self = shift;

    my $row = $self->dbh->select_row(<<SQL, $self->id);
SELECT * FROM items WHERE id = ?
SQL

    $self->$_($row->{$_}) for qw/name effect_type/;
    $self->effect_parameter($json->decode($row->{effect_parameter}));
}

my %effect_type_map = (
    1 => "MyApp::Model::ItemEffect::RecoverEnergy"
);

sub effector {
    my $self = shift;

    my $klass = $effect_type_map{$self->effect_type};
    return $klass->new_by_parameter($self->effect_parameter);
}

1;
