package MyApp::Model::User;
use strict;
use warnings;

use MyApp::Model::User;
use MyApp::DB;
use Data::UUID;

my $uuid = Data::UUID->new;

use Mouse;

extends "MyApp::Model::Base";

has id => (
    is => "rw",
    isa => "Int",
);

has name => (
    is => "rw",
    isa => "Str",
);

has avatar_id => (
    is => "rw",
    isa => "Int",
);

has energy => (
    is => "rw",
    isa => "Int",
);

has session => (
    is => "rw",
    isa => "Str",
);

no Mouse;

sub new_by_user_id {
    my ($class, $user_id) = @_;

    my $user = $class->new(id => $user_id);
    $user->_fetch;

    return $user;
}


sub new_by_session {
    my ($class, $session) = @_;

    my $user = $class->new(session => $session);
    $user->_fetch_by_session;

    return $user;
}

sub build {
    my ($self, %args) = @_;
    my $name = $args{name};
    my $avatar_id = $args{avatar_id};

    my $session = $self->_generate_session;
    $self->dbh->query(<<SQL, $name, $avatar_id, $session);
INSERT INTO `users` (`name`, `avatar_id`, `session`) VALUES (?, ?, ?);
SQL

    my $id = $self->dbh->last_insert_id;
    $self->id($id);
}

sub _generate_session {
    my $self = shift;
    return $uuid->create_str;
}

sub info {
    my $self = shift;

    $self->_fetch;

    return {
        user => {
            id => $self->id,
            name => $self->name,
            avatar_id => $self->avatar_id,
            energy => $self->energy,
        },
    };
}

sub info_with_session {
    my $self = shift;

    return {
        %{$self->info},
        session => $self->session,
    };
}

sub _fetch {
    my $self = shift;

    return if $self->_fetched;

    my $row = $self->dbh->select_row(<<SQL, $self->id);
SELECT * FROM users WHERE id = ?;
SQL
    $self->$_($row->{$_}) for qw/id name avatar_id energy session/;
    $self->_fetched(1);
}

sub _refetch {
    my $self = shift;

    $self->_fetched(0);
    $self->_fetch;
}

sub _fetch_by_session {
    my $self = shift;

    return if $self->_fetched;

    my $row = $self->dbh->select_row(<<SQL, $self->session);
SELECT * FROM users WHERE session = ?;
SQL
    $self->__update_attr($row);
    $self->_fetched(1);
}

sub __update_attr {
    my ($self, $row) = @_;

    $self->$_($row->{$_}) for qw/id name avatar_id energy session/;
}

sub new_by_hashref {
    my ($class, $hashref) = @_;

    my $user = $class->new(
        (map { $_ => $hashref->{$_} } qw/id name avatar_id energy session/),
        _fetched => 1,
    );
    return $user;
}

sub lock {
    my $self = shift;

    my $row = $self->dbh->select_row(<<SQL, $self->id);
SELECT * FROM users WHERE id = ? FOR UPDATE;
SQL

    $self->__update_attr($row);
}

sub set_energy {
    my ($self, $energy) = @_;

    $self->dbh->query(<<SQL, $energy, $self->id);
UPDATE `users` SET energy = ? WHERE id = ?;
SQL

    $self->_refetch;
}

1;
