package MyApp::Test::DB;
use strict;
use warnings;

use DBI;
use DBIx::Inspector;
use MyApp::DB::Schema;
use MyApp::DB;
use Test::More;
use parent "Exporter";

our @EXPORT = qw/dbtest/;

sub prepare {
    my ($class, $mysqld) = @_;

    my $dbh = DBI->connect($mysqld->dsn(dbname => "test"));
    my $ddl = MyApp::DB::Schema->output;
    my @ddls = split /\n/, $ddl;
    @ddls = grep { !/\A--/ } @ddls;
    $ddl = join "", @ddls;
    $dbh->do($_) for split /;/, $ddl;
}

sub init {
    my $class = shift;

    my $dsn = $class->dsn;
    MyApp::DB->overwrite_dsn($dsn, "", "");
}

sub dsn {
    return $ENV{PERL_TEST_MYSQLPOOL_DSN};
}

sub dbtest :prototype($&) {
    my ($note, $code) = @_;

    truncate_all();
    subtest $note => $code;
}

sub truncate_all {
    my $dbh = MyApp::DB->get;

    my $inspector = DBIx::Inspector->new(dbh => $dbh);
    my @tables = $inspector->tables;
    for my $table (@tables) {
        my $table_name = $table->name;
        $dbh->do(qq{TRUNCATE $table_name});
    }
}

1;
