use strict;
use warnings;
use Test2::V0;
use MyApp::Test::DB;

MyApp::Test::DB->init;

use MyApp;
use Plack::Test;
use HTTP::Request::Common;
use JSON;
use DBIx::FixtureLoader;
use FindBin qw/$Bin/;

my $json = JSON->new->utf8;

dbtest "friend scenario" => sub {
    my $app = MyApp->new->build;
    my $test = Plack::Test->create($app);

    my @resps;
    for my $name ("alice", "bob") {
        my $resp = $test->request(
            POST "/user",
            [name => $name, avatar_id => 1],
        );
        is $resp->code, "200";
        my $result = $json->decode($resp->content);
        push @resps, $result;
    }
    my ($alice, $bob) = @resps;

    my $bob_user_id = $bob->{user}->{id};
    subtest "friend request" => sub {
        my $resp = $test->request(
            POST "/user/$bob_user_id/friend",
            [session => $alice->{session}],
        );
        is $resp->code, "200";
    };

    subtest "friends list" => sub {
        my $resp = $test->request(
            GET "/user/$bob_user_id/friends",
            [session => $alice->{session}],
        );
        is $resp->code, "200";
        my $result = $json->decode($resp->content);

        like $result, hash {
            field friends => array {
                item hash {
                    id => $alice->{user}->{id},
                    name => $alice->{user}->{name},
                };
            };
        };
    };
};

dbtest "item scenario" => sub {
    my $app = MyApp->new->build;
    my $test = Plack::Test->create($app);

    my $loader = DBIx::FixtureLoader->new(dbh => MyApp::DB->get);
    $loader->load_fixture("$Bin/../master-data/csv/items.csv");

    my $user_resp = $test->request(
        POST "/user",
        [name => "alice", avatar_id => 1],
    );
    is $user_resp->code, "200";
    my $alice = $json->decode($user_resp->content);

    my $alice_user_id = $alice->{user}->{id};

    subtest "add item" => sub {
        my $resp = $test->request(
            POST "/my_user/item/add",
            [session => $alice->{session}, item_id => 2, num => 3],
        );
        is $resp->code, "200";
        my $result = $json->decode($resp->content);

        like $result, hash {
            field user_item => hash {
                item_id => 2,
                having_num => 3,
            };
        };
    };

    subtest "consume item" => sub {
        my $resp = $test->request(
            POST "/my_user/item/consume",
            [session => $alice->{session}, item_id => 2, num => 2],
        );
        is $resp->code, "200";
        my $result = $json->decode($resp->content);

        like $result, hash {
            field user => hash {
                energy => 200,
            };
            field user_item => hash {
                item_id => 2,
                having_num => 1,
            };
        };
    };
};

done_testing;
